﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Imaging;
using System.IO;

namespace howto_brighten_saturated_pixels
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private enum PixelTypes
        {
            Red,
            Green,
            Blue,
        }

        private Bitmap OriginalImage = null;

        private void mnuFileOpen_Click(object sender, EventArgs e)
        {
            if (ofdFile.ShowDialog() == DialogResult.OK)
            {
                OriginalImage = LoadBitmapUnlocked(ofdFile.FileName);
                picResult.Image = OriginalImage;
                picResult.Visible = true;
                mnuFileSaveAs.Enabled = true;
                btnApply.Enabled = true;
            }
        }

        private void mnuFileSaveAs_Click(object sender, EventArgs e)
        {
            if (sfdFile.ShowDialog() == DialogResult.OK)
            {
                SaveImage(picResult.Image, sfdFile.FileName);
            }
        }

        private void btnApply_Click(object sender, EventArgs e)
        {
            PixelTypes pixel_type = PixelTypes.Red;
            if (radGreen.Checked) pixel_type = PixelTypes.Green;
            else if (radBlue.Checked) pixel_type = PixelTypes.Blue;

            int diff = int.Parse(txtRedDiff.Text);
            float red_scale = float.Parse(txtRedScale.Text);
            picResult.Image = BrightenPixels(OriginalImage,
                pixel_type, diff, red_scale);
        }

        private Bitmap BrightenPixels(
            Bitmap bitmap, PixelTypes pixel_type,
            int threshold, float scale)
        {
            Bitmap bm = new Bitmap(bitmap);
            Bitmap32 bm32 = new Bitmap32(bm);
            bm32.LockBitmap();

            int width = bm.Width;
            int height = bm.Height;
            for (int y = 0; y < height; y++)
            {
                for (int x = 0; x < width; x++)
                {
                    byte r, g, b, a;
                    bm32.GetPixel(x, y, out r, out g, out b, out a);

                    bool over_threshold = false;
                    switch (pixel_type)
                    {
                        case PixelTypes.Red:
                            if ((r >= g + threshold) && (r > b + threshold))
                            {
                                int new_r = (int)(r * scale);
                                if (new_r > 255) new_r = 255;
                                else if (new_r < 0) new_r = 0;
                                r = (byte)new_r;
                            }
                            break;
                        case PixelTypes.Green:
                            if ((g >= r + threshold) && (g > b + threshold))
                            {
                                int new_g = (int)(g * scale);
                                if (new_g > 255) new_g = 255;
                                else if (new_g < 0) new_g = 0;
                                g = (byte)new_g;
                            }
                            break;
                        case PixelTypes.Blue:
                            if ((b >= r + threshold) && (b > g + threshold))
                            {
                                int new_b = (int)(b * scale);
                                if (new_b > 255) new_b = 255;
                                else if (new_b < 0) new_b = 0;
                                b = (byte)new_b;
                            }
                            break;
                    }

                    bm32.SetPixel(x, y, r, g, b, a);
                }
            }

            bm32.UnlockBitmap();
            return bm;
        }

        // Load a bitmap without locking it.
        private Bitmap LoadBitmapUnlocked(string file_name)
        {
            using (Bitmap bm = new Bitmap(file_name))
            {
                return new Bitmap(bm);
            }
        }

        // Save the file with the appropriate format.
        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }
    }
}
